#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <sys/time.h>
#include <math.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <fcntl.h>

#include "pntypes.h"
#include "pnresult.h"
#include "engtypes.h"

#define MAX_PATH 1024

//////////////////////////////////////////////////////////
// Global parameters
//////////////////////////////////////////////////////////

// this stores the duration info of the simulated file
static ULONG32 g_duration = 0;

// these store the current state while generating simulated frames
static double  g_audiobuffer_duration = 0.0;
static ULONG32 g_audiobuffer_size = 0;
static double  g_next_audio_time = 0.0;

int ossdev;

PN_RESULT SetupAudioCapture(
		    ULONG32 duration,
		
		    ULONG32 audio_sample_rate,
		    ULONG32 bits_ps,
		    ULONG32 num_channels,
		    ULONG32 audiobuffer_size)
{

  // set up audio device

  if ( ( ossdev = open("/dev/dsp", O_RDONLY) )  < 0 ) return PNR_FAIL; ;
  int tmp= num_channels - 1;   
   if ( ioctl(ossdev,SNDCTL_DSP_STEREO,&tmp) == -1)
     fprintf(stderr,"Failed to set number of channels\n"); 
   
   tmp=bits_ps; 
   if ( ioctl(ossdev,SNDCTL_DSP_SAMPLESIZE,&tmp) == -1)
     fprintf(stderr,"Failed to set sample size\n");
   tmp=audio_sample_rate; 
   if (ioctl(ossdev,SNDCTL_DSP_SPEED, &tmp) ==-1)
     fprintf(stderr,"Failed to set speed\n");


   g_audiobuffer_size = audiobuffer_size;
   g_duration = duration;

   // calculate the approximate size of an audio buffer in ms.
   // A buffer is nAudioBufferSize bytes long.  Using the audio format 
   // info, we can calculate the length in ms.
   {
     ULONG32 bytes_ps     =  bits_ps / 8;
     ULONG32 samples_one_chan  = audiobuffer_size / bytes_ps;
     ULONG32 samples_all_chan = samples_one_chan / num_channels;
     double  buffer_length_sec  = double(samples_all_chan) /
       double(audio_sample_rate);
     
	g_audiobuffer_duration = buffer_length_sec * 1000.0;
    }

    // initialize our variables for generating timestamps.

    g_next_audio_time = 0.0;

}



PN_RESULT GetAudio( ULONG32 *ptimestamp,
		    char **ppbuffer,
		    INT32 *pbuffer_size,
		    long *pbytes)
{
  PN_RESULT res = PNR_OK;
  long readbytes;

  unsigned char *buf = new unsigned char[g_audiobuffer_size];
  
  if(buf == NULL)
    res = PNR_OUTOFMEMORY;
  else
    {
      // do Audio grabbing here;
      
      readbytes = read(ossdev, buf, g_audiobuffer_size);
      
      // return the buffer
      *ppbuffer = (char *)buf;
      *pbuffer_size = g_audiobuffer_size;
      *ptimestamp = (ULONG32)g_next_audio_time;
      *pbytes = (long)readbytes;

      // update timestamp for next time
      g_next_audio_time += g_audiobuffer_duration;
    }
  
  return res;
}

void closeaudio() {
  close(ossdev);
}
