#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <sys/time.h>

#include "realencode.h"
#include "audiograb.h"
#include "vorbis.h"



char *dllpath = NULL;

int samplerate = 22050; // samples/sec
int bps        = 16;    // bits
int channels   =  1;    // [1=mono, 2=stereo]


int bitrate    = 32000;  // vorbis only
char *vorbisfile = NULL;
char *vorbisserver = NULL;
char *vorbispass = NULL;
int vorbisport = 8000;
char *vorbismount = NULL;

long audiobuffer_size =0;

//long audio_buffer_size = 8192;



char *realfile = NULL;
char *realserver = NULL;
int realport = 4040;
char *realuser = NULL;
char *realpass = NULL;
int audio_type = ENC_AUDIO_CONTENT_VOICE;
char *author = NULL;
char *title = NULL;
char *copyright = NULL;

RealEncode *re = new RealEncode();

void getargs(int argc, char** argv);
void usage(char *progname);


int main(int argc, char** argv)
{

  //getargs(argc, argv);

  if (argc < 2)
    usage(argv[0]);

  getargs(argc, argv);

  PN_RESULT result = PNR_OK;

  //RealEncode *re = new RealEncode();

  VorbisEncode *ve = new VorbisEncode();
  ve->SetupEncode(channels, bitrate, samplerate, ve->of, vorbisfile);

  re->initialize_dlls(dllpath);
  //re->initialize_dlls();
  re->create_build_eng();
  re->enumerate_pins();
  re->create_media_sample();
  //re->set_build_eng_properties();
  re->set_build_eng_properties(TRUE,   // do audio 
			       FALSE,  // do video
			       TRUE,   // do realtime
			       TRUE);  // do multirate

  re->set_clip_properties( realfile, 
			   realserver, 
			   realuser,
			   realpass,
			   realport,
			   title,
			   author,
			   copyright);


  re->set_misc_properties(audio_type, //audio content type
			  0,  //video_content -- not needed here
			  6); // player compatability
  re->set_audio_properties(samplerate, channels, bps);
  re->prepare_build_engine();
  audiobuffer_size = 4096; //re->get_suggested_audio_buffer();


  re->register_error_sink();

  // Vorbis stuff: Connect, ShoutInit
  //fprintf(stderr, "%s\n%d\n%s\n%s\n", vorbisserver, vorbisport, vorbispass, vorbismount);

  ve->ShoutInit(vorbisserver, vorbisport, vorbispass, vorbismount);
  if ( ( ve->Connect() ) == -1) {
    fprintf(stderr, "couldn't connect to icecast server, exiting...");
    exit(1);
  }

  //////////////////////////////////////////////////////////
  //
  // Encode!
  //
  //////////////////////////////////////////////////////////

  if(SUCCEEDED(result))
    result = SetupAudioCapture( 60*60*1000,  // time to encode in milleseconds - 1 hour
				samplerate,
				bps,
				channels,
				audiobuffer_size);
  
  //
  // When capturing live data, you must pass it to the build
  // engine interleaved.
  //
  
  while(SUCCEEDED(result))
    {
      char *buffer = NULL;
      ULONG32 timestamp;
      INT32 buffer_size;
      long bytes=0;
      
      result = GetAudio( &timestamp,
			 &buffer,
			 &buffer_size,
			 &bytes);
      //vorbis stuff

      if (bytes !=0) {
	ve->Encode(buffer, bytes,channels);
	//fprintf(stderr, "bytes = %d", bytes);
      }
      if(SUCCEEDED(result) && buffer == NULL)
	{
	  printf("End of transmission.\n");
	  break;
	}
      
      if(SUCCEEDED(result))
	
	result = re->pSample->SetBuffer( buffer,
					 buffer_size,
					 timestamp,
					 0);
      
      // Encode it!

      
      if(SUCCEEDED(result))
	result = re->pAudioPin->Encode(re->pSample);
      

      
      delete [] buffer;
      
      // Very important: if there is an asynchronous error
      // while encoding, our CMyErrorSink object will have
      // set g_bErrorOccurred.  If this happened, we need
      // to stop encoding.  So we set our 'res' variable
      // to the error that occurred and this will break
      // our while loop.
      //

	//if(SUCCEEDED(result) && g_bErrorOccurred)
	//{
	//  result = g_nError;
	//}
    }


    // encoding is finished -- cleanup
    re->end_real_encode();
    ve->CleanUp();
    closeaudio();    
    return 0;
}

void getargs(int argc, char** argv) {
  char tmpchar;
  int n,i=0;
  int ta_num =0;

  fprintf(stderr, "\n\n");

  if (argc < 2)  usage(argv[0]);

  while( (n=getopt(argc,argv,"D:c:r:F:S:P:U:X:T:A:C:t:a:f:s:p:x:m:b:")) != EOF)
    {
      switch(n) {
      case 'D': 
	dllpath = optarg;
	break;
      case 'c': 
	channels = atoi(optarg);
	break;
      case 'r': 
	samplerate = atoi(optarg);
	break;

      case 'F':
	realfile = optarg;
	break;
      case 'S':
	realserver = optarg;
	break;
      case 'P':
	realport = atoi(optarg);
	break;
      case 'U':
	realuser = optarg;
	break;
      case 'X':
	realpass = optarg;
	break;
      case 'T': 
	title = optarg;
	break;
      case 'A':
	author = optarg;
	break;
      case 'C':
	copyright = optarg;
	break;

	//vorbis options
      case 'f':
	vorbisfile = optarg;
	break;
      case 's':
	vorbisserver = optarg;
	break;
      case 'p':
	vorbisport = atoi(optarg);
	break;
      case 'x':
	vorbispass = optarg;
	break;
      case 'm':
	vorbismount = optarg;
	break;
      case 'b':
	bitrate = atoi(optarg);
	break;

      case 't':   //target audience
	i = 0;
	while ( (tmpchar = optarg[i++]) != NULL && ta_num < 10) {
	  switch(tmpchar) {
	  case '0':
	    re->ta[ta_num] = ENC_TARGET_28_MODEM;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_28_MODEM\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '1':
	    re->ta[ta_num] = ENC_TARGET_56_MODEM;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_56_MODEM\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '2':
	    re->ta[ta_num] = ENC_TARGET_SINGLE_ISDN;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_SINGLE_ISDN\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '3':
	    re->ta[ta_num] = ENC_TARGET_DUAL_ISDN;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_DUAL_ISDN\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '4':
	    re->ta[ta_num] = ENC_TARGET_LAN_LOW;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_LAN_LOW\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '5':
	    re->ta[ta_num] = ENC_TARGET_LAN_HIGH;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_LAN_HIGH\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '6':
	    re->ta[ta_num] = ENC_TARGET_256_DSL_CABLE;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_256_DSL_CABLE\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '7':
	    re->ta[ta_num] = ENC_TARGET_384_DSL_CABLE;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_384_DSL_CABLE\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  case '8':
	    re->ta[ta_num] = ENC_TARGET_512_DSL_CABLE;
	    fprintf(stderr, "target audience = %d, ENC_TARGET_512_DSL_CABLE\n", re->ta[ta_num]);
	    ta_num++;
	    break;
	  default:
	    fprintf(stderr, "target audience setting out of range, should be 0-8\n");
	    exit(1);
	    break;
	  }
	}
	break;
       
      case 'a':
	tmpchar = optarg[0];
	switch(tmpchar) {
	case '0':
	  audio_type = ENC_AUDIO_CONTENT_VOICE;
	  fprintf(stderr, "audio type = %d, ENC_AUDIO_CONTENT_VOICE;\n", audio_type);
	  break;
	case '1':
	  audio_type = ENC_AUDIO_CONTENT_VOICE_BACKGROUND;
	  fprintf(stderr, "audio type = %d, ENC_AUDIO_CONTENT_VOICE_BACKGROUND\n", audio_type);
	  break;
	case '2':
	  audio_type = ENC_AUDIO_CONTENT_MUSIC;
	  fprintf(stderr, "audio type = %d, ENC_AUDIO_CONTENT_MUSIC\n", audio_type);
	  break;
	case '3':
	  audio_type = ENC_AUDIO_CONTENT_MUSIC_STEREO;
	  fprintf(stderr, "audio type = %d, ENC_AUDIO_CONTENT_MUSIC_STEREO\n", audio_type);
	  break;
	default:
	  fprintf(stderr, "audio type out of range, should be 0-3\n");
	  exit(1);
	  break;
	}
	break;
      }
    }
 
 fprintf(stderr, "\n\n");
}

void usage(char *progname) //t:a:T:A:C:o:v:
{
        fprintf(stderr, "ogg vorbis AND real encoder\n");
        fprintf(stderr, "Usage: %s [options] \n", progname);

        fprintf(stderr, "\ngeneral options:\n");
        fprintf(stderr, "   -D     path to your realproducerSDK bin\n");
        fprintf(stderr, "   -c     1 = mono, 2 = stereo\n");
        fprintf(stderr, "   -r     audio capture samplerate\n");


        fprintf(stderr, "\nREAL options:\n");
        fprintf(stderr, "   -F     filename (for local save and server stream)\n");
        fprintf(stderr, "   -S     realserver name or IP\n");
        fprintf(stderr, "   -P     realserver port (default 4040)\n");
        fprintf(stderr, "   -U     realserver user name\n");
        fprintf(stderr, "   -X     realserver password\n");
        fprintf(stderr, "   -T     \"title\" \n");
        fprintf(stderr, "   -A     \"author\"\n");
        fprintf(stderr, "   -C     \"copyright\" (kills phun)\n");
        fprintf(stderr, "   -t 0-8 target audience\n");
        fprintf(stderr, "          can take more than one arguments such as:\n");
        fprintf(stderr, "          -t 01234  for surestream settings\n");
        fprintf(stderr, "   -a 0-3 audio type\n");
        fprintf(stderr, "          0=voice, 1=voice&bkgrd, 2=music, 3=stereo music\n");

        fprintf(stderr, "\nOgg Vorbis options for Icecast 2.0:\n");
        fprintf(stderr, "   -f     vorbis filename (for local save and server stream)\n");
        fprintf(stderr, "   -s     server IP (must be an ip number)\n");
        fprintf(stderr, "   -p     server port (default 8000)\n");
        fprintf(stderr, "   -x     password\n");
        fprintf(stderr, "   -m     mountpoint\n");
        fprintf(stderr, "   -b     bitrate\n");
	exit(1);
}  


