#include <stdio.h>
#include <assert.h>
#include <unistd.h>
#include <sys/time.h>

#include "realencode.h"


RealEncode::RealEncode() {
  PN_RESULT res = PNR_OK;
  IRMABuildEngine *pBuildEngine = NULL;
  IRMAInputPin    *pAudioPin = NULL;
  IRMAMediaSample *pSample = NULL;
  ta[0] = 1;
  for (int x = 1; x < 9; x++)
    ta[x] = -1;
}


void  RealEncode::initialize_dlls(const char *path) {

    if(SUCCEEDED(RealEncode::res))
    {
        printf("Setting up DLL paths\n");
            
        char szDllPath[2048];   
        UINT32 ulNumChars = 0;
            
        ulNumChars += sprintf(szDllPath+ulNumChars, 
                        "%s=%s", "DT_Plugins", path) + 1;
        ulNumChars += sprintf(szDllPath+ulNumChars, 
                        "%s=%s", "DT_Codecs", path) + 1;
        ulNumChars += sprintf(szDllPath+ulNumChars, 
                        "%s=%s", "DT_EncSDK", path) + 1;
        ulNumChars += sprintf(szDllPath+ulNumChars, 
                        "%s=%s", "DT_Common", path) + 1;
        // terminator
        ulNumChars += sprintf(szDllPath+ulNumChars, "") + 1;

        // Now that we have created the path, we hand it to the build engine
        // by using the exported SetDLLAccessPath function.

        RealEncode::res = SetDLLAccessPath(szDllPath);
    }
}   


void RealEncode::create_build_eng() {
  if(SUCCEEDED(RealEncode::res))
    {
      printf("creating build engine\n");   
      RealEncode::res = RMACreateRMBuildEngine(&pBuildEngine);
    }
}

void RealEncode::enumerate_pins() {
  if(SUCCEEDED(RealEncode::res))
    {
      IUnknown* tempUnk = NULL;
      IRMAEnumeratorIUnknown* pPinEnum = NULL;
      
      // get the pin enumerator
      printf("getting pin enumerator\n");
      
      RealEncode::res = pBuildEngine->GetPinEnumerator(&pPinEnum);
      assert(SUCCEEDED(RealEncode::res));
      
      // enumerate the pins.	
      printf("getting first pin\n");
      
      PN_RESULT resEnum = PNR_OK;
      resEnum = pPinEnum->First(&tempUnk);
      
      char* outputTypeStr = new char[ENC_MAX_STR];
      while(SUCCEEDED(RealEncode::res) &&
	    SUCCEEDED(resEnum) && 
	    resEnum != PNR_ELEMENT_NOT_FOUND)
	{
	  IRMAInputPin* tempPin = NULL;
	  RealEncode::res = tempUnk->QueryInterface(IID_IRMAInputPin, (void**)&tempPin);
	  PN_RELEASE(tempUnk);
	  
	  if(SUCCEEDED(RealEncode::res))
	    RealEncode::res = tempPin->GetOutputMimeType(outputTypeStr, ENC_MAX_STR);
	  
	  if(SUCCEEDED(RealEncode::res))
	    {
	      if(strcmp(outputTypeStr, MIME_REALAUDIO) == 0)
		{
		  // save this pin in pAudioPin
		  pAudioPin = tempPin;
		  // addref it because we are going to use it later
		  pAudioPin->AddRef();
		}
	      /*else if(strcmp(outputTypeStr, MIME_REALVIDEO) == 0)
		{
		pVideoPin = tempPin;
		pVideoPin->AddRef();
		}
		printf("getting next pin\n");
	      */ 
	      PN_RELEASE(tempPin);
	      resEnum = pPinEnum->Next(&tempUnk);
	    } 
	  else 
	    {
	      printf("Cannot query input pin interface.\n");
	    }
	} 
      delete [] outputTypeStr;
      
      PN_RELEASE(pPinEnum);
    }
}


void RealEncode::create_media_sample() {

  if(SUCCEEDED(RealEncode::res))
    {
      printf("Creating a Media Sample\n");
      
      IRMABuildClassFactory*  pClassFactory = NULL;
      
      RealEncode::res = pBuildEngine->QueryInterface(IID_IRMABuildClassFactory,
					 (void**)&pClassFactory);
      
      if(SUCCEEDED(RealEncode::res))
	{
	  // create an instance of class IRMAMediaSample
	  
	  RealEncode::res = pClassFactory->CreateInstance(CLSID_IRMAMediaSample,
					      NULL, IID_IRMAMediaSample, 
					      (void **)&pSample);
	}
      
      PN_RELEASE(pClassFactory);
    }
}
 

void RealEncode::set_build_eng_properties(bool audio, 
					  bool video, 
					  bool realtime, 
					  bool multirate) 
{
  if(SUCCEEDED(RealEncode::res))
    {
      RealEncode::res = pBuildEngine->SetDoOutputMimeType(MIME_REALAUDIO, audio);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBuildEngine->SetDoOutputMimeType(MIME_REALVIDEO, video);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBuildEngine->SetDoOutputMimeType(MIME_REALEVENT, FALSE);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBuildEngine->SetDoOutputMimeType(MIME_REALIMAGEMAP, FALSE);
      
      // for live, we must encode in realtime.
            
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBuildEngine->SetRealTimeEncoding(realtime);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBuildEngine->SetDoMultiRateEncoding(multirate);
    }
}


void RealEncode::set_clip_properties(const char *filename, 
				     const char *host, 
				     const char *login,
				     const char *pass,
				     const int port,
				     const char *title,
				     const char *author,
				     const char *copyright) 
{

  if(SUCCEEDED(RealEncode::res))
    {
      IRMAClipProperties* pClipProps = NULL;
      
      // get the clip properties object
      RealEncode::res = pBuildEngine->GetClipProperties(&pClipProps);
      
      // we want to create a file
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetDoOutputFile(TRUE);
      
      // we do not want to stream live to a server
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetDoOutputServer(TRUE);
      
      // filename we want to create
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetOutputFilename(filename);
      
      // here is the server we want to stream to, and the
      // name of the file on that server.  If the server
      // is set up in the standard way, then you can play
      // it with this url in the player:
      // rtsp://192.168.69.7:554/encoder/test.rm
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetOutputServerInfo(
					      host,
					      filename,
					      port, //should be 4040
					      login,
					      pass);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetTitle(title);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetAuthor(author);
		    
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetCopyright(copyright);

      if(SUCCEEDED(res))
	res = pClipProps->SetSelectiveRecord(TRUE);

	PN_RELEASE(pClipProps);
    }
}
 
void RealEncode::set_clip_properties(const char *filename, 
				     const char *title,
				     const char *author,
				     const char *copyright) 
{

  if(SUCCEEDED(RealEncode::res))
    {
      IRMAClipProperties* pClipProps = NULL;
      
      // get the clip properties object
      RealEncode::res = pBuildEngine->GetClipProperties(&pClipProps);
      
      // we want to create a file
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetDoOutputFile(TRUE);
      
      // we do not want to stream live to a server
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetDoOutputServer(TRUE);
      
      // filename we want to create
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetOutputFilename(filename);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetTitle(title);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetAuthor(author);
		    
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pClipProps->SetCopyright(copyright);

      if(SUCCEEDED(res))
	res = pClipProps->SetSelectiveRecord(TRUE);

	PN_RELEASE(pClipProps);
    }
}

void RealEncode::set_misc_properties(int audio_content, 
				     int video_content,
				     int player_compat) 
{
  int i = 0;
  if(SUCCEEDED(RealEncode::res))
    {
      IRMABasicTargetSettings* pBasicSettings = NULL;
      IRMATargetSettings *pTargSettings = NULL;
      
      // Build Engine returns a TargetSettings object,
      // we must query for BasicTargetSettings
      RealEncode::res = pBuildEngine->GetTargetSettings(&pTargSettings);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pTargSettings->QueryInterface(IID_IRMABasicTargetSettings,
					    (void**)&pBasicSettings);
      PN_RELEASE(pTargSettings);
      
      // reset the target audiences
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBasicSettings->RemoveAllTargetAudiences();
      
      while (ta[i] != -1) {
	if(SUCCEEDED(RealEncode::res)) {
	  RealEncode::res = pBasicSettings->AddTargetAudience(ta[i]);  
	}
	i++;
      }
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBasicSettings->SetAudioContent(audio_content);
      //video quality
      if(SUCCEEDED(res))
	RealEncode::res = pBasicSettings->SetAudioContent(video_content);

      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pBasicSettings->SetPlayerCompatibility(player_compat); // 6
      
      PN_RELEASE(pBasicSettings);
    }
}

void RealEncode::set_audio_properties(int samplerate, 
				      int channels, 
				      int bps) 
{
  
  if(SUCCEEDED(RealEncode::res))
    {
	IRMAPinProperties* pPinProps = NULL;
	IRMAAudioPinProperties* pAudioPinProps = NULL;

	RealEncode::res = pAudioPin->GetPinProperties(&pPinProps);
			    
	if(SUCCEEDED(RealEncode::res))
	    RealEncode::res = pPinProps->QueryInterface(IID_IRMAAudioPinProperties,
					    (void**)&pAudioPinProps);

	PN_RELEASE(pPinProps);


	// Here, we configure the audio pin with the format
	// of the data that we will be passing to it.
	// These constants are defined at the top of this file.
	if(SUCCEEDED(RealEncode::res))
	    RealEncode::res = pAudioPinProps->SetSampleRate(samplerate);

	if(SUCCEEDED(RealEncode::res))
	    RealEncode::res = pAudioPinProps->SetNumChannels(channels);

	if(SUCCEEDED(RealEncode::res))
	    RealEncode::res = pAudioPinProps->SetSampleSize(bps);

    }
}
 
void RealEncode::set_video_properties(int width,
				      int height,
				      int framerate, 
				      int format) 
{
  if(SUCCEEDED(RealEncode::res))
    {
      IRMAPinProperties* pPinProps = NULL;
      IRMAVideoPinProperties* pVideoPinProps = NULL;
      
      RealEncode::res = pVideoPin->GetPinProperties(&pPinProps);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pPinProps->QueryInterface(IID_IRMAVideoPinProperties,
					(void**)&pVideoPinProps);
      
      PN_RELEASE(pPinProps);
      // Here, we configure the video pin with the video size and frame
      // rate, and video format.  This is the format in which we will
      // be providing video to the Encode() call.
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pVideoPinProps->SetFrameRate( framerate );
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pVideoPinProps->SetVideoSize(width, height);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pVideoPinProps->SetVideoFormat(format);

    }
}

void RealEncode::prepare_build_engine() 
{
  if(SUCCEEDED(RealEncode::res))
    {
      printf("calling PrepareToEncode()\n");

      RealEncode::res = pBuildEngine->PrepareToEncode();

    }
}

long RealEncode::get_suggested_audio_buffer() 
{
  printf("top of getting suggested input size\n");
  if(SUCCEEDED(RealEncode::res))
    {
      printf("getting suggested input size\n");
      
      UINT32 nSize = 0;
      
      IRMAAudioInputPin *pAudioSpecificPin = NULL;
      
      RealEncode::res = pAudioPin->QueryInterface(IID_IRMAAudioInputPin,
				      (void **)&pAudioSpecificPin);
      
      if(SUCCEEDED(RealEncode::res))
	RealEncode::res = pAudioSpecificPin->GetSuggestedInputSize(&nSize);
      
      if(SUCCEEDED(RealEncode::res))
	printf("Audio pin suggested input size = %d\n", nSize);
      
      PN_RELEASE(pAudioSpecificPin);

      return nSize;
    }
}

void RealEncode::register_error_sink()
{
  if(SUCCEEDED(RealEncode::res))
    {
      IUnknown *pMyErrorSinkObject = NULL;
      IRMAErrorSink *pErrorSink = NULL;
      IRMAErrorSinkControl *pErrorSinkControl = NULL;
      
      RealEncode::res = CMyErrorSink::CreateInstance(&pMyErrorSinkObject);
      
      if(SUCCEEDED(RealEncode::res))
	{
	  printf("ErrorSink: getting from MyErrorSink object\n");	  
	  RealEncode::res = pMyErrorSinkObject->QueryInterface(IID_IRMAErrorSink,
						   (void **)&pErrorSink);
	}
      
      if(SUCCEEDED(RealEncode::res))
	{
	  printf("ErrorSink: getting from Build Engine\n");
	  RealEncode::res = pBuildEngine->QueryInterface(IID_IRMAErrorSinkControl,
					     (void **)&pErrorSinkControl);
	}
      
      if(SUCCEEDED(RealEncode::res))
	{
	  printf("ErrorSink: adding error sink\n");
	  RealEncode::res = pErrorSinkControl->AddErrorSink(pErrorSink,
						PNLOG_EMERG, PNLOG_INFO);
	}
      
      printf("ErrorSink: done, res = 0x%08x\n", RealEncode::res);
      
      PN_RELEASE(pErrorSink);
      PN_RELEASE(pErrorSinkControl);
      PN_RELEASE(pMyErrorSinkObject);
    }
}


void RealEncode::end_real_encode() 
{
  
  if(SUCCEEDED(RealEncode::res))
    {
      printf("calling DoneEncoding()\n");
      RealEncode::res = pBuildEngine->DoneEncoding();
    }
  
  PN_RELEASE(pAudioPin);
  PN_RELEASE(pVideoPin);
  
  PN_RELEASE(pSample);
  PN_RELEASE(pBuildEngine);
  
  printf("Done Encoding, res = 0x%08x\n", RealEncode::res); 
  printf("Done, res = 0x%08x\n", RealEncode::res);
  
}

//////////////////////////////
// Error stuff
/////////////////////////////

BOOL g_bErrorOccurred = FALSE;
PN_RESULT g_nError = PNR_OK;

PN_RESULT CMyErrorSink::CreateInstance(IUnknown **ppUnk)
{
    CMyErrorSink *pObj = new CMyErrorSink();

    if(pObj == NULL)
    {
        *ppUnk = NULL;
        return PNR_OUTOFMEMORY;
    }
    else
    {
        *ppUnk = (IUnknown*)pObj;
        (*ppUnk)->AddRef();
        return PNR_OK;
    }
}

CMyErrorSink::CMyErrorSink()
: m_nRefCount(0)
{
}

CMyErrorSink::~CMyErrorSink()
{
}

STDMETHODIMP CMyErrorSink::QueryInterface(REFIID riid, void** ppvObj)
{
    if(!ppvObj)
        return PNR_POINTER;
    else if(IsEqualIID(IID_IUnknown, riid))
    {
        AddRef();
        *ppvObj = (IUnknown*)this;
        return PNR_OK;
    }
    else if(IsEqualIID(IID_IRMAErrorSink, riid))
    {
        AddRef();
        *ppvObj = (IRMAErrorSink*)this;
        return PNR_OK;
    }

    *ppvObj = NULL;
    return PNR_NOINTERFACE;
}

ULONG32 CMyErrorSink::AddRef()
{
    return InterlockedIncrement(&m_nRefCount);
}

ULONG32 CMyErrorSink::Release()
{
    if(InterlockedDecrement(&m_nRefCount) > 0)
        return m_nRefCount;
    
    delete this;
    return 0;
}

STDMETHODIMP CMyErrorSink::ErrorOccurred(const UINT8    unSeverity,  
                                const ULONG32   ulRMACode,
                                const ULONG32   ulUserCode,
                                const char*     pUserString,
                                const char*     pMoreInfoURL
                                )
{
    printf("ERROR OCCURRED: "
                "sev=%d, err=0x%08x, usercode=%d, "
                "string=\"%s\", moreinfo=\"%s\"\n",
                unSeverity,  
                ulRMACode,
                ulUserCode,
                pUserString,
                pMoreInfoURL);
    
    if(unSeverity <= PNLOG_ERR)
    {
        g_bErrorOccurred = TRUE;
        g_nError = ulRMACode;
    }
    return(PNR_OK);
}
