#ifndef _REALENCODE_H_
#define _REALENCODE_H_

#include "pntypes.h"
#include "pnwintyp.h"
#include "pnresult.h"
#include "pncom.h"
#include "rmaenum.h"
#include "rmapckts.h"
#include "rmmetain.h"
#include "rmaerror.h"
#include "engtypes.h"
#include "engtargs.h"
#include "progsink.h"
#include "rmbldeng.h"
#include "setdllac.h"

class RealEncode {
 public:

  RealEncode();

  PN_RESULT res;
  IRMABuildEngine *pBuildEngine;
  IRMAInputPin    *pAudioPin;
  IRMAInputPin    *pVideoPin;
  IRMAMediaSample *pSample;
  int ta[9];
  
  void initialize_dlls(const char *path);

  void create_build_eng();
  void enumerate_pins();

  void create_media_sample();

  void set_build_eng_properties(bool audio, 
				bool video, 
				bool realtime, 
				bool multirate);
  
  void set_clip_properties(const char *filename, 
			   const char *host, 
			   const char *login,
			   const char *pass,
			   const int port,
			   const char *title,
			   const char *author,
			   const char *copyright);
  
  void set_clip_properties(const char *filename, 
			   const char *title,
			   const char *author,
			   const char *copyright);
  
  void set_misc_properties(int audio_content, 
			   int video_content, 
			   int player_compat);

  void set_audio_properties(int samplerate, 
			    int channels, 
			    int bps);

  void set_video_properties(int width,
			    int height,
			    int framerate, 
			    int format);
 
  void prepare_build_engine();

  long get_suggested_audio_buffer();

  void register_error_sink();

  void end_real_encode();
};


class CMyErrorSink : public IRMAErrorSink
{
public:
    static PN_RESULT CreateInstance(IUnknown **pUnk);

    STDMETHOD(QueryInterface)   (THIS_ REFIID riid, void** ppvObj);
    STDMETHOD_(ULONG32,AddRef)  (THIS);
    STDMETHOD_(ULONG32,Release) (THIS);

    STDMETHOD(ErrorOccurred)    (THIS_
                                const UINT8     unSeverity,  
                                const ULONG32   ulRMACode,
                                const ULONG32   ulUserCode,
                                const char*     pUserString,
                                const char*     pMoreInfoURL
                                );
    
protected:
    UINT32 m_nRefCount;

    CMyErrorSink();
    ~CMyErrorSink();
};
#endif
